from discoPy.rest.user.user import UserData
from discoPy.rest.channel.channel import ChannelData
from discoPy.rest.guild.guild import GuildData
from discoPy.rest.stage.stage import StageData
from discoPy.rest.webhook.webhook import WebhookData
from discoPy.rest.application.application import ApplicationData

class User(UserData):
    pass
class Guild(GuildData):
    pass
class Channel(ChannelData):
    pass
class Stage(StageData):
    pass
class Webhook(WebhookData):
    pass
class Application(ApplicationData):
    pass
